with Ada.Containers.Indefinite_Vectors;

with GNAT.Sockets;

generic
   type Category_Enum is (<>);
   type Directory_Entry_Type (Category : Category_Enum) is private;
package P42.Broadcast.Directory is

   package Directory_Entry_Vectors is new Ada.Containers.Indefinite_Vectors
     (Positive, Directory_Entry_Type, "=");

   protected type Directory_Type (Local_Entry : access Directory_Entry_Type) is
      new Message_Handler_Interface with
      overriding
      procedure Receive_New_Message
        (Data : in Ada.Streams.Stream_Element_Array;
         Address : in GNAT.Sockets.Sock_Addr_Type);
      procedure Connect;
      procedure Close;
      function Get_Entries return Directory_Entry_Vectors.Vector;
      function Get_Entries (Category : in Category_Enum)
                           return Directory_Entry_Vectors.Vector;
      procedure Clear_Entries;
      function Get_Local_Entry return Directory_Entry_Type;
      procedure Ask_For (Category : in Category_Enum);
      procedure Ask_For_All;
      entry Wait_For_Answer;
      procedure Set_Listen;
   private
      Entries : Directory_Entry_Vectors.Vector;
      Socket : GNAT.Sockets.Socket_Type;
      Address : GNAT.Sockets.Sock_Addr_Type;
      Answer_Seen : Boolean := False;
   end Directory_Type;
   type Directory_Access is access all Directory_Type;

private
   type Demand_Enum is (Publish, Publish_All);
   for Demand_Enum'Size use 8;
   for Demand_Enum use (Publish => 1,
                        Publish_All => 2);

   type Message_Type (Kind : Message_Kind_Enum;
                      Category : Category_Enum) is record
      case Kind is
         when Demand =>
            What : Demand_Enum;
         when Publication =>
            Directory_Entry : Directory_Entry_Type (Category);
      end case;
   end record;

end P42.Broadcast.Directory;
