with Ada.Containers.Indefinite_Vectors;
with Ada.Containers.Vectors;

with GNAT.Sockets;

generic
   type Category_Enum is (<>);
   type Ressource_Type (Category : Category_Enum) is private;
   with function Get_Name (Resource : in Ressource_Type) return String;
package P42.Broadcast.Ressources is

   package Ressource_Vectors is new Ada.Containers.Indefinite_Vectors
     (Positive, Ressource_Type, "=");

   type Callback_Type is access procedure (Ressource : in out Ressource_Type;
                                           Address : GNAT.Sockets.Sock_Addr_Type);
   package Callback_Vectors is new Ada.Containers.Vectors (Positive,
                                                           Callback_Type,
                                                           "=");

   protected type Ressource_Shell_Type is
      new Message_Handler_Interface with
      overriding
      procedure Receive_New_Message
        (Data : in Ada.Streams.Stream_Element_Array;
         Address : in GNAT.Sockets.Sock_Addr_Type);
      procedure Connect;
      procedure Close;
      function Get_Ressources return Ressource_Vectors.Vector;
      function Get_Ressource (Name : in String) return Ressource_Type;
      procedure Clear;
      procedure Ask_For (Name : in String;
                         Category : in Category_Enum);
      entry Wait_For_Answer;
      procedure Set_Listen;
      procedure Append (Ressource : Ressource_Type);
      procedure Remove (Ressource_Name : String);
      procedure Subscribe (Callback : Callback_Type);
   private
      Entries : Ressource_Vectors.Vector;
      Local_Ressources : Ressource_Vectors.Vector;
      Socket : GNAT.Sockets.Socket_Type;
      Address : GNAT.Sockets.Sock_Addr_Type;
      Answer_Seen : Boolean := False;
      Callbacks : Callback_Vectors.Vector;
   end Ressource_Shell_Type;
   type Ressource_Shell_Access is access all Ressource_Shell_Type;

   Ressource_Not_Found : exception;

private

   subtype Name_Length_Range is Natural range 0 .. 128;

   type Message_Type (Kind : Message_Kind_Enum;
                      Category : Category_Enum;
                      Name_Length : Name_Length_Range) is record
      case Kind is
         when Demand =>
            Name : String (1 .. Name_Length);
         when Publication =>
            Ressource : Ressource_Type (Category);
      end case;
   end record;

end P42.Broadcast.Ressources;
