with Ada.Exceptions;
with Ada.Text_IO;

package body P42.Broadcast is

   -------------------
   -- Listener_Task --
   -------------------

   use type Ada.Streams.Stream_Element_Offset;

   task body Listener_Task is
      Socket : GNAT.Sockets.Socket_Type;
      Address : GNAT.Sockets.Sock_Addr_Type;
      From : GNAT.Sockets.Sock_Addr_Type;
      Data : Ada.Streams.Stream_Element_Array (1 .. MAX_DATA_PAYLOAD);
      Last : Ada.Streams.Stream_Element_Offset;
      use GNAT.Sockets;
      Message_Handler : Message_Handler_Access;
   begin
      loop
         select
            accept Start (Handler : Message_Handler_Access) do
               Message_Handler := Handler;
            end Start;
            Address.Port := Group_Port;
            Address.Addr := Inet_Addr (Image => Group);
            Create_Socket (Socket, Family_Inet, Socket_Datagram);
            Set_Socket_Option
              (Socket,
               IP_Protocol_For_IP_Level,
               (Multicast_TTL, 1));
            Set_Socket_Option
              (Socket,
               IP_Protocol_For_IP_Level,
               (Multicast_Loop, True));
            Set_Socket_Option
              (Socket,
               IP_Protocol_For_IP_Level,
               (Add_Membership,
                Inet_Addr (Group),
                Any_Inet_Addr));
            Set_Socket_Option
              (Socket,
               Socket_Level,
               (Reuse_Address, True));
            Set_Socket_Option
              (Socket,
               Socket_Level,
               (Receive_Timeout, 1.0));
            Bind_Socket (Socket, Address);
         or
            terminate;
         end select;
         loop
            select
               accept Stop;
               Close_Socket (Socket);
               exit;
            else
               begin
                  Receive_Socket (Socket, Data, Last, From);
                  Message_Handler.Receive_New_Message
                    (Data (Data'First .. Last), From);
               exception
                  when GNAT.Sockets.Socket_Error =>
                     null;
               end;
            end select;
         end loop;
      end loop;
   exception
      when E : others =>
         Ada.Text_IO.Put_Line ("listener exception : " &
                                 Ada.Exceptions.Exception_Name (E));
         Ada.Text_IO.Put_Line ("           message : " &
                                 Ada.Exceptions.Exception_Message (E));
   end Listener_Task;

   procedure Send_Message (Socket : in GNAT.Sockets.Socket_Type;
                           Address : in GNAT.Sockets.Sock_Addr_Type;
                           Data : in Ada.Streams.Stream_Element_Array) is
      Last : Ada.Streams.Stream_Element_Offset;
   begin
      GNAT.Sockets.Send_Socket (Socket,
                                Data,
                                Last,
                                Address);
      if Last /= Data'Last then
         raise Multicast_IO_Error with "send message failed" & Last'Img &
           "/=" & Data'Last'Img;
      end if;
   end Send_Message;

end P42.Broadcast;
