with Ada.Streams;

with GNAT.Sockets;
generic
   Group : in String;
   Group_Port : in GNAT.Sockets.Port_Type;
package P42.Broadcast is

   type Message_Handler_Interface is synchronized interface;
   procedure Receive_New_Message
     (Message_Handler : in out Message_Handler_Interface;
      Data : in Ada.Streams.Stream_Element_Array;
      Address : in GNAT.Sockets.Sock_Addr_Type) is abstract;
   type Message_Handler_Access is access all Message_Handler_Interface'Class;

   task type Listener_Task is
      entry Start (Handler : Message_Handler_Access);
      entry Stop;
   end Listener_Task;

   MAX_DATA_PAYLOAD : constant := 4_096;

   procedure Send_Message (Socket : in GNAT.Sockets.Socket_Type;
                           Address : in GNAT.Sockets.Sock_Addr_Type;
                           Data : in Ada.Streams.Stream_Element_Array);

   Multicast_IO_Error : exception;
private

   Demand_Network_Representation : constant := 1;
   Publication_Network_Representation : constant := 2;

   type Message_Kind_Enum is (Demand, Publication);
   for Message_Kind_Enum'Size use Standard'Storage_Unit;
   for Message_Kind_Enum use
     (Demand => Demand_Network_Representation,
      Publication => Publication_Network_Representation);
end P42.Broadcast;
