--  Package used to read configuration file automatically
--  @version 0.0.9

with Ada.Text_IO;
with Ada.Containers.Vectors;

package P42.CFD is

   generic
      type Element_Type is private;
   package Configuration_Pkg is

      package Element_Vectors is new Ada.Containers.Vectors (Positive,
                                                             Element_Type,
                                                             "=");
      type Parse_File_Interface is interface;
      procedure Parse_File (Parser : in out Parse_File_Interface;
                            File : Ada.Text_IO.File_Type) is abstract;
      function Get_Elements (Parser : Parse_File_Interface)
                            return Element_Vectors.Vector is abstract;

      generic
         type Parser_Type is new Parse_File_Interface with private;
         type Key_Type is private;
         with function Get_Key (Element : Element_Type) return Key_Type;
      package Configuration_Repository_Pkg is

         Key_Not_Found : exception;

         protected type Repository_Type is
            procedure Build (File_Name : String);
            procedure Clear;
            function Get_Elements return Element_Vectors.Vector;
            function Get_Element_By_Key (Key : Key_Type) return Element_Type;
         private
            Elements : Element_Vectors.Vector;
         end Repository_Type;

      end Configuration_Repository_Pkg;
   end Configuration_Pkg;

end P42.CFD;
