--  Convert an unbounded_string in integer whether it is a decimal or hexadecimal value
--  @version 1.0.0

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package P42.Convert.Characters_2_Numeric is
   --  Test if a character is a numeric value
   --  @param Val : A Character
   function Is_Numeric (Val : Character) return Boolean;

   generic
      type T_Number is mod <>;
   package Generic_Char2Num is
      --  Convert an unbounded_string hexadecimal in integer value
      --  @param Val : An Unbounded_String
      --  @exception P42.Convert.Wrong_Type : Thrown when Val is not a valid hexadecimal argument
      function Read_Hex (Val : Unbounded_String) return T_Number;

      --  Convert an unbounded_string decimal in integer value
      --  @param Val : An Unbounded_String
      --  @exception P42.Convert.Wrong_Type : Thrown when Val is not a valid decimal argument
      function Read_Dec (Val : Unbounded_String) return T_Number;

      --  Convert an unbounded_string number in integer value
      --  @param Val : An Unbounded_String
      --  @exception P42.Convert.Wrong_Type : Thrown when Val is not a valid number argument
      function Read_Hex_Dec (Val : Unbounded_String) return T_Number;
   private
      --  Convert an unbounded_string hexadecimal in integer value
      --  @param Val : An Unbounded_String
      --  @exception Constraint_Error : Thrown when Val doesn't begin with "0x" or "0X"
      function Read_Hex_Interne (Val : Unbounded_String) return T_Number;

      --  Convert an unbounded_string decimal in integer value
      --  @param Val : An Unbounded_String
      --  @exception P42.Convert.Wrong_Type : Thrown when Val is not a valid decimal argument
      function Read_Dec_Interne (Val : Unbounded_String) return T_Number;
   end Generic_Char2Num;
end P42.Convert.Characters_2_Numeric;
