with GNAT.Expect;
with GNAT.OS_Lib;

package body P42.Gnuplot is

   use GNAT.Expect;
   --  taken from g-expect.adb
   function Waitpid (Pid : Process_Id) return Integer;
   pragma Import (C, Waitpid, "__gnat_waitpid");
   --  Wait for a specific process id, and return its exit code

   --------------
   -- Generate --
   --------------

   function To_String (Date : Ada.Calendar.Time) return String;

   procedure Generate
     (Datas : Data_Type;
      File_Name : String := "";
      Title : String := "";
      Xlabel : String := "";
      Ylabel : String := "";
      Zlabel : String := "")
   is
      Descriptor : Process_Descriptor;
      Null_Args : constant GNAT.OS_Lib.Argument_List (1 .. 0) :=
        (others => null);
      One_Arg : GNAT.OS_Lib.Argument_List (1 .. 1) := (1 => new String'("-p"));
      Exit_Status : Integer;
   begin
      if File_Name = "" then
         Non_Blocking_Spawn (Descriptor => Descriptor,
                             Command => "/usr/bin/gnuplot",
                             Args => One_Arg,
                             Buffer_Size => 0,
                             Err_To_Out => True);
      else
         Non_Blocking_Spawn (Descriptor => Descriptor,
                             Command => "/usr/bin/gnuplot",
                             Args => Null_Args,
                             Buffer_Size => 0,
                             Err_To_Out => True);
         Send (Descriptor, "set terminal png",
               Empty_Buffer => True);
         Send (Descriptor, "set output """ & File_Name & """",
               Empty_Buffer => True);
      end if;
      if Title = "" then
         Send (Descriptor, "set title 'no title'",
               Empty_Buffer => True);
      else
         Send (Descriptor, "set title '" & Title & "'",
               Empty_Buffer => True);
      end if;
      if Xlabel = "" then
         Send (Descriptor, "set xlabel 'X'",
               Empty_Buffer => True);
      else
         Send (Descriptor, "set xlabel '" & Xlabel & "'",
               Empty_Buffer => True);
      end if;
      if Ylabel = "" then
         Send (Descriptor, "set ylabel 'Y'",
               Empty_Buffer => True);
      else
         Send (Descriptor, "set ylabel '" & Ylabel & "'",
               Empty_Buffer => True);
      end if;
      if Zlabel = "" then
         Send (Descriptor, "set zlabel 'Z'",
               Empty_Buffer => True);
      else
         Send (Descriptor, "set zlabel '" & Zlabel & "'",
               Empty_Buffer => True);
      end if;
      case Datas.Dimension is
         when Timed =>
            Send (Descriptor, "set xdata time", Empty_Buffer => True);
            Send (Descriptor, "set timefmt ""%Y/%m/%dT%H:%M:%S""", Empty_Buffer => True);
            Send (Descriptor, "plot '-' using 1:2", Empty_Buffer => True);
            for I in Datas.Timed_Points'Range loop
               declare
                  X_String : constant String := To_String (Datas.Timed_Points (I).Date);
               begin
                  Send (Descriptor, X_String & Image (Datas.Timed_Points (I).Y), Empty_Buffer => True);
               end;
            end loop;
         when Two_D =>
            Send (Descriptor, "plot '-'", Empty_Buffer => True);
            for I in Datas.Two_D_Points'Range loop
               Send (Descriptor,
                     Image (Datas.Two_D_Points (I).X) & " " &
                       Image (Datas.Two_D_Points (I).Y),
                     Empty_Buffer => True);
            end loop;
         when Three_D =>
            Send (Descriptor, "set hidden3d",
                  Empty_Buffer => True);
            Send (Descriptor, "set grid",
                  Empty_Buffer => True);
            Send (Descriptor, "splot '-'",
                  Empty_Buffer => True);
            for I in Datas.Three_D_Points'Range loop
               Send (Descriptor,
                     Image (Datas.Three_D_Points (I).X) & " " &
                       Image (Datas.Three_D_Points (I).Y) & " " &
                       Image (Datas.Three_D_Points (I).Z),
                     Empty_Buffer => True);
            end loop;
      end case;
      Send (Descriptor, "e", Empty_Buffer => True);
      Send (Descriptor, "quit", Empty_Buffer => True);
      GNAT.OS_Lib.Free (One_Arg (1));
      Exit_Status := Waitpid (Descriptor.Get_Pid);
      if Exit_Status /= 0 then
         raise Gnuplot_Execution_Error
           with "non zero exit status :" & Exit_Status'Img;
      end if;
      Close (Descriptor);
   end Generate;

   function To_String (Date : Ada.Calendar.Time) return String is
      Year : constant Ada.Calendar.Year_Number := Ada.Calendar.Year (Date);
      Month : constant Ada.Calendar.Month_Number := Ada.Calendar.Month (Date);
      Day : constant Ada.Calendar.Day_Number := Ada.Calendar.Day (Date);
      First_Date_String : String := Year'Img & Month'Img & Day'Img;
      Seconds : constant Ada.Calendar.Day_Duration := Ada.Calendar.Seconds (Date);
      Truncated_Seconds : constant Natural := Natural (Seconds);
      Hour : constant Natural := Truncated_Seconds / 3600;
      Minutes : constant Natural := (Truncated_Seconds - 3600 * Hour) / 60;
      Last_Seconds : constant Natural := Truncated_Seconds - 3600 * Hour - 60 * Minutes;
      Second_Date_String : String := Hour'Img & Minutes'Img & Last_Seconds'Img;
   begin
      for I in First_Date_String'First + 1 .. First_Date_String'Last loop
         if First_Date_String (I) = ' ' then
            First_Date_String (I) := '/';
         end if;
      end loop;
      Second_Date_String (Second_Date_String'First) := 'T';
      for I in Second_Date_String'First + 1 .. Second_Date_String'Last loop
         if Second_Date_String (I) = ' ' then
            Second_Date_String (I) := ':';
         end if;
      end loop;
      declare
         Result : constant String := First_Date_String & Second_Date_String;
      begin
         return Result (Result'First + 1 .. Result'Last);
      end;
   end To_String;

end P42.Gnuplot;
