with Ada.Calendar;

generic
   type Base_Type is private;
   with function Image (Item : Base_Type) return String;
package P42.Gnuplot is

   Maximum_Points : constant := 65536;
   subtype Point_Number_Type is Positive range 1 .. Maximum_Points;

   type Dimension_Enum is (Timed, Two_D, Three_D);

   type Two_D_Point_Type is record
      X : Base_Type;
      Y : Base_Type;
   end record;
   type Timed_Point_Type is record
      Date : Ada.Calendar.Time;
      Y : Base_Type;
   end record;
   type Three_D_Point_Type is record
      X : Base_Type;
      Y : Base_Type;
      Z : Base_Type;
   end record;
   type Two_D_Point_Array is array (Point_Number_Type range <>) of Two_D_Point_Type;
   type Timed_Point_Array is array (Point_Number_Type range <>) of Timed_Point_Type;
   type Three_D_Point_Array is array (Point_Number_Type range <>) of Three_D_Point_Type;

   type Data_Type (Dimension : Dimension_Enum;
                   Point_Nb : Point_Number_Type) is record
      case Dimension is
         when Timed =>
            Timed_Points : Timed_Point_Array (1 .. Point_Nb);
         when Two_D =>
            Two_D_Points : Two_D_Point_Array (1 .. Point_Nb);
         when Three_D =>
            Three_D_Points : Three_D_Point_Array (1 .. Point_Nb);
      end case;
   end record;

   procedure Generate (Datas : Data_Type;
                       File_Name : String := "";
                       Title : String := "";
                       Xlabel : String := "";
                       Ylabel : String := "";
                       Zlabel : String := "");

   Gnuplot_Execution_Error : exception;

end P42.Gnuplot;
