package body P42.Html_Utils is

   use Ada.Strings.Unbounded;

   -----------------
   -- Append_Code --
   -----------------

   procedure Append_Code
     (Obj : in out T_Page_Html;
      Code : String;
      LF : Boolean := True)
   is
   begin
      Obj.Body_Page := Obj.Body_Page & Code;
      if LF then
         Obj.Body_Page := Obj.Body_Page & ASCII.LF;
      end if;
   end Append_Code;

   -----------------
   -- Append_Code --
   -----------------

   procedure Append_Code
     (Obj : in out T_Page_Html;
      Code : Ada.Strings.Unbounded.Unbounded_String;
      LF : Boolean := True)
   is
   begin
      Obj.Body_Page := Obj.Body_Page & Code;
      if LF then
         Obj.Body_Page := Obj.Body_Page & ASCII.LF;
      end if;
   end Append_Code;

   ------------------------
   -- Append_Code_Tagged --
   ------------------------

   procedure Append_Code_Tagged
     (Obj : in out T_Page_Html;
      Tag : String;
      Code : String;
      LF : Boolean := True)
   is
   begin
      Append_Code (Obj, "<" & Tag & ">" & Code & "</" & Tag & ">", LF);
   end Append_Code_Tagged;

   -----------------
   -- Append_Head --
   -----------------

   procedure Append_Head
     (Obj : in out T_Page_Html;
      Code : Ada.Strings.Unbounded.Unbounded_String;
      LF : Boolean := True)
   is
   begin
      Obj.Header := Obj.Header & Code;
      if LF then
         Obj.Header := Obj.Header & ASCII.LF;
      end if;
   end Append_Head;

   procedure Append_Head
     (Obj : in out T_Page_Html;
      Code : String;
      LF : Boolean := True)
   is
   begin
      Obj.Header := Obj.Header & Code;
      if LF then
         Obj.Header := Obj.Header & ASCII.LF;
      end if;
   end Append_Head;

   ----------
   -- Head --
   ----------

   procedure Head
     (Obj : in out T_Page_Html;
      Title : String;
      CSS_URI : String := "";
      Refresh : Duration := 0.0;
      Url : String := "")
   is
   begin
      Obj.Header := Obj.Header & "<title>" & Title & "</title>" & ASCII.LF;
      if Refresh /= 0.0 then
         if Url = "" then
            Obj.Header := Obj.Header & "<meta http-equiv=""refresh"" content=""" & Refresh'Img & """>" & ASCII.LF;
         else
            Obj.Header := Obj.Header & "<meta http-equiv=""refresh"" content=""" & Refresh'Img
              & "; url=" & Url & """>" & ASCII.LF;
         end if;
      end if;
      if CSS_URI /= "" then
         Obj.Header := Obj.Header & "<LINK rel=""stylesheet"" type=""text/css"" href=""" & CSS_URI & """>" & ASCII.LF;
      end if;
   end Head;

   procedure Malformed_URI (Obj : in out T_Page_Html;
                            Sub_URI : String) is
   begin
      Obj.Header := Null_Unbounded_String & "<title>Malformed URI</title>";
      Obj.Body_Page := Null_Unbounded_String & "<b><font color=""Red"">Malformed URI : " &
        Sub_URI & "</font></b>";
   end Malformed_URI;


   function Return_Page (Obj : T_Page_Html) return Unbounded_String is
      Build : Unbounded_String;
   begin
      Build := "<html><head>" & ASCII.LF & Obj.Header & "</head><body>" & ASCII.LF
        & Obj.Body_Page & "</body></html>";
      return Build;
   end Return_Page;
end P42.Html_Utils;
