package body P42.Images.Convert is

   function To_Fits (Image : Tiff.Image_Type) return Fits.Image_Type is
      Image_To_Return : Fits.Image_Type;
      use type Tiff.Image_Kind_Enum;
   begin
      if Image.Kind /= Tiff.Grayscale then
         raise Unsupported_Feature with Image.Kind'Img & " can't be converted for the moment";
      end if;
      Image_To_Return.Naxis := 2;
      Image_To_Return.Naxis1 := Image.Rows.First_Element.Nb_Elements;
      Image_To_Return.Naxis2 := Positive (Image.Rows.Length);
      Image_To_Return.Bitpix := Float;
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "SIMPLE",
                                                        Is_Value => True,
                                                        Value => "T",
                                                        Comment => "File Does Conform To FITS Standard"));
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "BITPIX",
                                                        Is_Value => True,
                                                        Value => "-32",
                                                        Comment => "comes from greyscale tiff image"));
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "NAXIS",
                                                        Is_Value => True,
                                                        Value => "2"));
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "NAXIS1",
                                                        Is_Value => True,
                                                        Value => Image_To_Return.Naxis1'Img));
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "NAXIS2",
                                                        Is_Value => True,
                                                        Value => Image_To_Return.Naxis2'Img));
      Image_To_Return.Keywords.Append (Fits.To_Keyword (Name => "END"));
      declare
         Data : Fits.Data_Row_Vectors.Vector;
         use type Tiff.Sample_Format_Enum;
      begin
         for Row of Image.Rows loop
            if Row.Sample_Format_Value /= P42.Images.Tiff.Ieee_Float then
               raise Unsupported_Feature with Row.Sample_Format_Value'Img & " should be Ieee_Float";
            end if;
            declare
               New_Row : Fits.Data_Row_Type (Kind => Float, Length => Row.Ieee_Float_Data'Length);
            begin
               New_Row.Floats := Row.Ieee_Float_Data;
               Data.Append (New_Row);
            end;
         end loop;
         Image_To_Return.Datas.Append (Data);
      end;
      return Image_To_Return;
   end To_Fits;

end P42.Images.Convert;
