with P42.Images.Fits;

package P42.Images.Tools
  with Preelaborate => True is

   use type Interfaces.IEEE_Float_32;

   function Get_Min_Max (Matrix : Float_Matrix) return Float_Array;
   function Step (Min, Max : Interfaces.IEEE_Float_32;
                  Nb_Bin : Positive) return Interfaces.IEEE_Float_32 is
      ((Min + Max) / Interfaces.IEEE_Float_32 (Nb_Bin));

   function Get_Distribution (Matrix : Float_Matrix;
                              Min : Interfaces.IEEE_Float_32;
                              Max : Interfaces.IEEE_Float_32;
                              Nb_Bin : Positive;
                              Exclude_Min : Boolean := False) return Longlong_Array;

   function Get_Masses (Matrix : Float_Matrix;
                        Max_Cluster_Number : Positive) return Longlong_Array;
   function Get_Statistics (Matrix : Float_Matrix;
                            Masses : Longlong_Array) return Float_Matrix;

   function Calcul_Distribution (Data : Longlong_Array) return Longlong_Array;

   function Compress_Distribution (Distribution : Longlong_Array;
                                   Compression_Factor : Positive) return Longlong_Array
     with Pre => Compression_Factor >= 2;

   procedure Split_With_Interval (Base : in out Fits.Image_Type;
                                  Depth : Positive;
                                  Min : Interfaces.IEEE_Float_32;
                                  Max : Interfaces.IEEE_Float_32;
                                  Default_Value : Interfaces.IEEE_Float_32)
     with Pre => Depth <= Natural (Base.Datas.Length) and
                 Base.Bitpix = Float;

   function Identify_Clusters (Base : Fits.Image_Type;
                               Depth : Positive;
                               Background : Interfaces.IEEE_Float_32) return Fits.Data_Matrix_Type
     with Pre => Depth <= Natural (Base.Datas.Length) and
                 Base.Bitpix = Float;

end P42.Images.Tools;
