--------------------------------------------------------------------------
--                                                                      --
--           Copyright: Copyright (C) 2000-2010 CNRS/IN2P3              --
--                                                                      --
-- Narval framework is free  software; you can redistribute  it and/or  --
-- modify  it   under  terms  of  the  GNU General  Public  License as  --
-- published  by  the  Free Software Foundation; either version  2, or  --
-- (at your option) any later version. Narval framework is distributed  --
-- in the hope  that  they will  be useful, but  WITHOUT ANY WARRANTY;  --
-- without even the implied warranty of  MERCHANTABILITY or FITNESS FOR --
-- A PARTICULAR PURPOSE. See the  GNU. General Public License for more  --
-- details. You should have received  a copy of the GNU General Public  --
-- License distributed with Narval; see file COPYING. If not, write to  --
-- the Free Software  Foundation,  Inc., 51 Franklin St,  Fifth Floor,  --
-- Boston, MA 02110-1301 USA.                                           --
--------------------------------------------------------------------------

with System;

package P42.Shared_Library is

   pragma Preelaborate;

   --  The MODE argument to `dlopen' contains one of the following:
   RTLD_LAZY         : constant := 16#00000001#;
   RTLD_NOW          : constant := 16#00000002#;
   RTLD_BINDING_MASK : constant := 16#00000003#;
   RTLD_NOLOAD       : constant := 16#00000004#;
   RTLD_DEEPBIND     : constant := 16#00000008#;

   --  If the following bit is set in the MODE argument to `dlopen',
   --  the symbols of the loaded object and its dependencies are made
   --  visible as if the object were linked directly into the program.
   RTLD_GLOBAL : constant := 16#00000100#;

   --  Unix98 demands the following flag which is the inverse to RTLD_GLOBAL.
   --  The implementation does this by default and so we can define the
   --  value to zero.
   RTLD_LOCAL : constant := 16#00000000#;

   --  Do not delete object when closed.
   RTLD_NODELETE : constant := 16#00001000#;

   subtype Handle_Type is System.Address; --  handle en francais :-)
   subtype Flag_Type is Integer;
   function Open_Library (Library_Name : String;
                          Options : Flag_Type) return Handle_Type;
   function Library_Error return String;
   generic
      type Symbol_Access is private;
   function Library_Symbol (Library : Handle_Type;
                            Symbol_Name : String)
                           return Symbol_Access;

   function Close_Library (Library : Handle_Type) return Integer;
   Library_Loading_Failed : exception;
   Unknown_Symbol : exception;
end P42.Shared_Library;
