--  code based on
--  c1af110c7b6b733fd6f486024da7a47fe865d85c221718141a879b768cafa71c  fips180-2withchangenotice.pdf

with Ada.Streams;

with P42.Sha_Constants;

generic
   type Unsigned_Type is mod <>;
   type Unsigned_Array is array (Natural range <>) of Unsigned_Type;
   Mod_Padding : Unsigned_Type;
   Ref_Padding : Unsigned_Type;
   Bit_Rotation_Matrix : Sha_Constants.Rotation_Matrix;
   K : Unsigned_Array;
   H_Init : Unsigned_Array;
   with function Rotate_Right (Value : Unsigned_Type; Amount : Natural) return Unsigned_Type;
   with function Shift_Right (Value : Unsigned_Type; Amount : Natural) return Unsigned_Type;
   with function Shift_Left (Value : Unsigned_Type; Amount : Natural) return Unsigned_Type;
   with function To_Big_Endian (Item : Unsigned_Type) return Unsigned_Type;
   Cut_Output_Size : Natural := 0;
package P42.ShaXXX is

   use Ada.Streams;

   type Context_Type is limited private;

   procedure Hash_Computation (Context : out Context_Type;
                               Message : Stream_Element_Array);

   procedure Hash_Computation (Context : out Context_Type;
                               File_Name : String);

   function Image (Context : Context_Type) return String;

   task type Hash_Computation_Task is
      entry Start (File_Name : String);
      entry Get_Result (Context : out Context_Type);
   end Hash_Computation_Task;

   type Hash_Computation_Task_Array is array (Positive range <>) of Hash_Computation_Task;

private

   subtype Buffer_Array is Ada.Streams.Stream_Element_Array (1 .. 32_768 * Unsigned_Type'Size);

   type Context_Type is record
      Ctx : Unsigned_Array (0 .. 7);
   end record;


   function Ch (X, Y, Z : Unsigned_Type) return Unsigned_Type is
      ((X and Y) xor ((not X) and Z));

   function Maj (X, Y, Z : Unsigned_Type) return Unsigned_Type is
      ((X and Y) xor (X and Z) xor (Y and Z));

   function Sum_0 (X : Unsigned_Type) return Unsigned_Type is
      (Rotate_Right (X, Bit_Rotation_Matrix (1, 1)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (1, 2)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (1, 3)));

   function Sum_1 (X : Unsigned_Type) return Unsigned_Type is
      (Rotate_Right (X, Bit_Rotation_Matrix (2, 1)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (2, 2)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (2, 3)));

   function Sigma_0 (X : Unsigned_Type) return Unsigned_Type is
      (Rotate_Right (X, Bit_Rotation_Matrix (3, 1)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (3, 2)) xor
         Shift_Right (X, Bit_Rotation_Matrix (3, 3)));

   function Sigma_1 (X : Unsigned_Type) return Unsigned_Type is
      (Rotate_Right (X, Bit_Rotation_Matrix (4, 1)) xor
         Rotate_Right (X, Bit_Rotation_Matrix (4, 2)) xor
         Shift_Right (X, Bit_Rotation_Matrix (4, 3)));

end P42.ShaXXX;

