with Ada.Command_Line;
with Ada.Containers;
with Ada.Strings.Unbounded;
with Ada.Text_IO;
with Interfaces;

with GNAT.Command_Line;

with P42.Images.Convert;
with P42.Images.Fits;
with P42.Images.Tiff;

procedure Analyse_Tiff is
   Options : constant String := "help file= output=";
   use type Ada.Containers.Count_Type;
   Input_File : Ada.Strings.Unbounded.Unbounded_String;
   Output_File : Ada.Strings.Unbounded.Unbounded_String;
   use Ada.Strings.Unbounded;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'f' =>
            Input_File := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'h' =>
            Ada.Text_IO.Put_Line (Ada.Command_Line.Command_Name & " {-help|-file=file_name.tiff}");
         when 'o' =>
            Output_File := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if Input_File /= Null_Unbounded_String then
      declare
         Image : constant P42.Images.Tiff.Image_Type := P42.Images.Tiff.Image (To_String (Input_File));
         Min : Interfaces.IEEE_Float_32 := Interfaces.IEEE_Float_32'Last;
         Max : Interfaces.IEEE_Float_32 := Interfaces.IEEE_Float_32'First;
         use type P42.Images.Tiff.Sample_Format_Enum;
         use type Interfaces.IEEE_Float_32;
      begin
         Ada.Text_IO.Put_Line (Image.Header.Endianness'Img);
         Ada.Text_IO.Put_Line (Image.Kind'Img);
         Ada.Text_IO.Put_Line ("IFD_OFFSET : " & Image.Header.IFD_Offset'Img);
         Ada.Text_IO.Put_Line ("directories :" & Image.Directories.Length'Img);
         if Image.Directories.Length < 16 then
            for Directory_Entry of Image.Directories loop
               Ada.Text_IO.Put_Line ("tag " & Directory_Entry.Sub_Directory.Tag'Img);
               Ada.Text_IO.Put_Line ("count" & Directory_Entry.Sub_Directory.Count'Img);
               Ada.Text_IO.Put ("kind " & Directory_Entry.Kind'Img);
               case Directory_Entry.Kind is
                  when P42.Images.Short =>
                     for A_Short of Directory_Entry.Shorts loop
                        Ada.Text_IO.Put_Line (A_Short'Img);
                     end loop;
                  when P42.Images.Long =>
                     Ada.Text_IO.Put_Line (Directory_Entry.A_Long'Img);
                  when P42.Images.Ascii =>
                     Ada.Text_IO.New_Line;
                     for A_String of Directory_Entry.Strings loop
                        Ada.Text_IO.Put_Line (A_String);
                     end loop;
                  when others =>
                     Ada.Text_IO.New_Line;
               end case;
            end loop;
         end if;
         for Row of Image.Rows loop
            if Row.Sample_Format_Value = P42.Images.Tiff.Ieee_Float then
               for I in Row.Ieee_Float_Data'Range loop
                  if Row.Ieee_Float_Data (I) < Min then
                     Min := Row.Ieee_Float_Data (I);
                  end if;
                  if Row.Ieee_Float_Data (I) > Max then
                     Max := Row.Ieee_Float_Data (I);
                  end if;
               end loop;
            end if;
         end loop;
         Ada.Text_IO.Put_Line ("min" & Min'Img);
         Ada.Text_IO.Put_Line ("max" & Max'Img);
         if Output_File /= Null_Unbounded_String then
            declare
               Fits_Image : constant P42.Images.Fits.Image_Type := P42.Images.Convert.To_Fits (Image);
            begin
               P42.Images.Fits.Write (Fits_Image, To_String (Output_File));
            end;
         end if;
      end;
   end if;
end Analyse_Tiff;
