with Ada.Text_IO;

with GNAT.Command_Line;

with Vegetable_Trade;

procedure Directory_Multicast is
   Options : constant String := "delay= listen category= vegetable=";
   Listen : Boolean := False;
   Category : Vegetable_Trade.Category_Enum :=
     Vegetable_Trade.Buyer;
   Vegetable : Vegetable_Trade.Vegetable_Enum := Vegetable_Trade.Potato;
   Wait_Time : Duration := 10.0;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'c' =>
            Category := Vegetable_Trade.Category_Enum'Value
              (GNAT.Command_Line.Parameter);
         when 'd' =>
            Wait_Time := Duration'Value (GNAT.Command_Line.Parameter);
         when 'l' =>
            Listen := True;
         when 'v' =>
            Vegetable := Vegetable_Trade.Vegetable_Enum'Value
              (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   declare
      use Vegetable_Trade;
      Directory_Entry : aliased Directory_Entry_Type (Category);
      Directory : aliased Vegetable_Directory.Directory_Type
        (Directory_Entry'Access);
      Listener : Broadcast_Directory.Listener_Task;
   begin
      Directory.Connect;
      Directory_Entry.Vegetable := Vegetable;
      case Category is
         when Wholesale =>
            Directory_Entry.Number_Per_Sale := 5;
            Directory_Entry.Reduced_Price := 1.0;
         when Retail_Sale =>
            Directory_Entry.Price := 1.2;
         when Buyer =>
            null;
      end case;
      if Listen then
         Listener.Start (Directory'Unchecked_Access);
      end if;
      Directory.Ask_For_All;
      Directory.Ask_For (Vegetable_Trade.Wholesale);
      delay Wait_Time;
      declare
         Entries : constant Vegetable_Directory.Directory_Entry_Vectors.Vector
           := Directory.Get_Entries;
      begin
         Ada.Text_IO.Put_Line ("directory length :" &
                                 Entries.Length'Img);
         for An_Entry of Entries loop
            Ada.Text_IO.Put_Line (An_Entry.Vegetable'Img & " " &
                                    An_Entry.Category'Img);
         end loop;
      end;
      if Listen then
         Listener.Stop;
      end if;
      Directory.Close;
   end;
end Directory_Multicast;
