with Ada.Text_IO;
with Ada.Strings.Unbounded;

with GNAT.Command_Line;

with Vegetable_Trade;

procedure Ressources_Multicast is
   Options : constant String := "name= vegetable= ask= kind=";
   Vegetable : Vegetable_Trade.Vegetable_Enum := Vegetable_Trade.Potato;
   use Ada.Strings.Unbounded;
   Name : Unbounded_String := To_Unbounded_String ("AGATA");
   Ask : Unbounded_String := To_Unbounded_String ("charlotte");
   Ask_Vegetable : Vegetable_Trade.Vegetable_Enum := Vegetable_Trade.Potato;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'a' =>
            Ask := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'n' =>
            Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'v' =>
            Vegetable := Vegetable_Trade.Vegetable_Enum'Value
              (GNAT.Command_Line.Parameter);
         when 'k' =>
            Ask_Vegetable := Vegetable_Trade.Vegetable_Enum'Value
              (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   declare
      use Vegetable_Trade;
      Ressource : Ressource_Type (Vegetable);
      Ressource_Shell : aliased Vegetable_Ressource.Ressource_Shell_Type;
      Listener : Broadcast_Ressource.Listener_Task;
   begin
      Ressource.Name := Name;
      Ressource_Shell.Append (Ressource);
      Ressource_Shell.Connect;
      Listener.Start (Ressource_Shell'Unchecked_Access);
      delay 2.0;
      Ressource_Shell.Ask_For (To_String (Ask), Ask_Vegetable);
      delay 2.0;
      declare
         Entries : constant Vegetable_Ressource.Ressource_Vectors.Vector
           := Ressource_Shell.Get_Ressources;
      begin
         Ada.Text_IO.Put_Line ("ressources found :" &
                                 Entries.Length'Img);
         for An_Entry of Entries loop
            Ada.Text_IO.Put_Line (An_Entry.Kind'Img & " " &
                                    To_String (An_Entry.Name));
         end loop;
      end;
      Listener.Stop;
      Ressource_Shell.Close;
   end;
end Ressources_Multicast;
