with Ada.Command_Line;
with Ada.Exceptions;
with Ada.Text_IO;

with GNAT.Traceback.Symbolic;

with P42.Sha256;

procedure Sha256sum is
   Context : P42.Sha256.Context_Type;
begin
   if Ada.Command_Line.Argument_Count = 1 then
      P42.Sha256.Hash_Computation (Context, Ada.Command_Line.Argument (1));
      Ada.Text_IO.Put_Line (P42.Sha256.Image (Context) & "  " & Ada.Command_Line.Argument (1));
   else
      declare
         Hash_Tasks : P42.Sha256.Hash_Computation_Task_Array (1 .. Ada.Command_Line.Argument_Count);
      begin
         for I in Hash_Tasks'Range loop
            Hash_Tasks (I).Start (Ada.Command_Line.Argument (I));
         end loop;
         for I in Hash_Tasks'Range loop
            Hash_Tasks (I).Get_Result (Context);
            Ada.Text_IO.Put_Line (P42.Sha256.Image (Context) & "  " & Ada.Command_Line.Argument (I));
         end loop;
      end;
   end if;
exception
   when E : others =>
      Ada.Text_IO.Put_Line ("unexpected exception : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("                       " & Ada.Exceptions.Exception_Message (E));
      Ada.Text_IO.Put_Line (GNAT.Traceback.Symbolic.Symbolic_Traceback (E));
end Sha256sum;
