with Ada.Strings.Unbounded;

with P42.Broadcast.Directory;
pragma Elaborate_All (P42.Broadcast.Directory);
with P42.Broadcast.Ressources;
pragma Elaborate_All (P42.Broadcast.Ressources);

package Vegetable_Trade is

   type Vegetable_Enum is (Potato, Tomato);
   for Vegetable_Enum'Size use 8;
   for Vegetable_Enum use (Potato => 1,
                           Tomato => 2);

   type Category_Enum is (Wholesale, Retail_Sale, Buyer);
   for Category_Enum'Size use 8;
   for Category_Enum use (Wholesale => 1,
                          Retail_Sale => 2,
                          Buyer => 3);

   type Directory_Entry_Type (Category : Category_Enum) is record
      Vegetable : Vegetable_Enum;
      case Category is
         when Wholesale =>
            Number_Per_Sale : Natural;
            Reduced_Price : Float;
         when Retail_Sale =>
            Price : Float;
         when Buyer =>
            null;
      end case;
   end record;

   type Ressource_Type (Kind : Vegetable_Enum) is record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Number_Of_Ressource_Available : Natural := 100;
      case Kind is
         when Potato =>
            Cleaned : Boolean := False;
         when Tomato =>
            Grounded : Boolean := True;
      end case;
   end record;

   function Get_Name (Ressource : Ressource_Type) return String;

   Directory_Group : constant String := "234.42.0.42";
   Directory_Group_Port : constant := 4242;

   package Broadcast_Directory is new P42.Broadcast (Directory_Group,
                                                     Directory_Group_Port);

   package Vegetable_Directory is new Broadcast_Directory.Directory
     (Category_Enum, Directory_Entry_Type);

   Ressource_Group : constant String := "234.0.42.42";
   Ressource_Group_Port : constant := 4243;

   package Broadcast_Ressource is new P42.Broadcast (Ressource_Group,
                                                     Ressource_Group_Port);

   package Vegetable_Ressource is new Broadcast_Ressource.Ressources
     (Vegetable_Enum, Ressource_Type, Get_Name);

end Vegetable_Trade;
