with Ada.Strings.Unbounded;

with GNAT.Command_Line;

with P42.Images.Fits;

procedure P42.Tools.Extract_Fits is
   Options : constant String := "fits= pattern= output=";
   Fits_File : Ada.Strings.Unbounded.Unbounded_String;
   Pattern_Image : Ada.Strings.Unbounded.Unbounded_String;
   Output_Name : Ada.Strings.Unbounded.Unbounded_String;
   use Ada.Strings.Unbounded;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'f' =>
            Fits_File := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'o' =>
            Output_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Pattern_Image := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if Fits_File = Null_Unbounded_String or
     Pattern_Image = Null_Unbounded_String or
     Output_Name = Null_Unbounded_String
   then
      raise Program_Error with "fits, pattern and output options have to be given";
   end if;
   declare
      Fits_Image : constant Images.Fits.Image_Type := Images.Fits.Read_Image (To_String (Fits_File));
      Pattern : constant Images.Selection_Pattern := Images.To_Pattern (To_String (Pattern_Image));
      New_Image : Images.Fits.Image_Type := Images.Fits.Extract_Images (Fits_Image, Pattern);
   begin
      Images.Fits.Append_Keyword (New_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "extract_fits"));
      Images.Fits.Append_Keyword (New_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "fits parameter " & To_String (Fits_File)));
      Images.Fits.Append_Keyword (New_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "pattern parameter " & To_String (Pattern_Image)));
      Images.Fits.Append_Keyword (New_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "output parameter " & To_String (Output_Name)));
      Images.Fits.Write (New_Image, To_String (Output_Name));
   end;
end P42.Tools.Extract_Fits;
