with Ada.Strings.Unbounded;

with GNAT.Command_Line;

with P42.Images.Fits;

procedure P42.Tools.Merge_Fits is
   Options : constant String := "fits1= fits2= output=";
   First_Image_Name : Ada.Strings.Unbounded.Unbounded_String;
   Second_Image_Name : Ada.Strings.Unbounded.Unbounded_String;
   Output_Name : Ada.Strings.Unbounded.Unbounded_String;
   use Ada.Strings.Unbounded;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'f' =>
            if GNAT.Command_Line.Full_Switch = "fits1" then
               First_Image_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
            elsif GNAT.Command_Line.Full_Switch = "fits2" then
               Second_Image_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
            else
               raise Program_Error;
            end if;
         when 'o' =>
            Output_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if First_Image_Name = Null_Unbounded_String or
     Second_Image_Name = Null_Unbounded_String or
     Output_Name = Null_Unbounded_String
   then
      raise Program_Error with "fits1, fits2 and output options have to be given";
   end if;
   declare
      First_Image : Images.Fits.Image_Type := Images.Fits.Read_Image (To_String (First_Image_Name));
      Second_Image : constant Images.Fits.Image_Type :=
        Images.Fits.Read_Image (To_String (Second_Image_Name));
   begin
      Images.Fits.Append (First_Image, Second_Image);
      Images.Fits.Append_Keyword (First_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "merge_fits"));
      Images.Fits.Append_Keyword (First_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "fits1 parameter " &
                                                            To_String (First_Image_Name)));
      Images.Fits.Append_Keyword (First_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "fits2 parameter " &
                                                            To_String (Second_Image_Name)));
      Images.Fits.Append_Keyword (First_Image,
                                  Images.Fits.To_Keyword ("HISTORY",
                                                          Is_Value => False,
                                                          Value => "output parameter " & To_String (Output_Name)));
      Images.Fits.Write (First_Image, To_String (Output_Name));
   end;
end P42.Tools.Merge_Fits;
